// file      : Gate.cpp
// author    : Boris Kolpackov <boris@dre.vanderbilt.edu>
// cvs-id    : $Id$

#include "Gate.h"

#include <iostream>

using std::cerr;
using std::endl;

using namespace CORBA;
using namespace CosNotifyComm;
using namespace CosNotification;
using namespace CosNotifyChannelAdmin;

Gate::
Gate (ACE_INET_Addr const& group, EventChannel_ptr ch)
    : socket_ (group, false)
{
  // Obtain proxy consumer.
  //
  ProxyConsumer_var pc (
    ch->default_supplier_admin ()->obtain_notification_push_consumer (
      STRUCTURED_EVENT, consumer_id_));

  consumer_ = StructuredProxyPushConsumer::_narrow (pc.in ());

  consumer_->connect_structured_push_supplier (
    StructuredPushSupplier::_nil ());

  // Register as consumer.
  //
  StructuredPushConsumer_var ref (_this ()); // Activate on default POA.

  ProxySupplier_var ps (
    ch->default_consumer_admin ()->obtain_notification_push_supplier (
      STRUCTURED_EVENT, supplier_id_));

  supplier_ = StructuredProxyPushSupplier::_narrow (ps.in ());

  supplier_->connect_structured_push_consumer (ref.in ());

  //
  //
  if (ACE_OS::thr_create (&tracker_thunk,
                          this,
                          THR_JOINABLE,
                          &thread_) != 0) ::abort ();
}

ACE_THR_FUNC_RETURN Gate::
tracker_thunk (void* arg)
{
  Gate* a (reinterpret_cast<Gate*> (arg));
  a->tracker ();
  return 0;
}

void Gate::
tracker ()
{
  char buf[2048];

  while (true)
  {
    size_t n (socket_.recv (buf, 2048));

    TAO_InputCDR cdr (buf, n);

    StructuredEvent e;

    cdr >> e;

    // Add ttl.
    //
    ULong i (0);

    for (ULong i (0); i < e.header.variable_header.length (); ++i)
    {
      if (strcmp (e.header.variable_header[i].name, "ttl") == 0) break;
    }

    if (i == e.header.variable_header.length ())
    {
      e.header.variable_header.length (i + 1);

      e.header.variable_header[i].name = string_dup("ttl");
    }

    e.header.variable_header[i].value <<= ULong (1);

    cerr << "IN: "
         << e.header.fixed_header.event_type.domain_name << "::"
         << e.header.fixed_header.event_type.type_name << " "
         << e.header.fixed_header.event_name << endl;

    consumer_->push_structured_event (e);
  }
}

void Gate::
push_structured_event (StructuredEvent const& e) throw ()
{
  for (ULong i (0); i < e.header.variable_header.length (); ++i)
  {
    if (strcmp (e.header.variable_header[i].name, "ttl") == 0)
    {
      ULong ttl;

      e.header.variable_header[i].value >>= ttl;

      if (ttl <= 1)
      {
        cerr << "DROPPED" << endl;
        return;
      }


      break;
    }
  }

  cerr << "OUT: "
       << e.header.fixed_header.event_type.domain_name << "::"
       << e.header.fixed_header.event_type.type_name << " "
       << e.header.fixed_header.event_name << endl;

  TAO_OutputCDR cdr;

  cdr << e;

  size_t size (cdr.total_length ());

  if (size > 2048) ::abort ();

  char buffer[2048];

  {
    char* buf (buffer);

    for (ACE_Message_Block const* mb = cdr.begin ();
         mb != 0;
         mb = mb->cont ())
    {
      ACE_OS::memcpy (buf, mb->rd_ptr (), mb->length ());
      buf += mb->length ();
    }
  }

  socket_.send (buffer, size);
}


void Gate::
disconnect_structured_push_consumer () throw ()
{
  // We don't care.
}

void Gate::
offer_change (EventTypeSeq const&, EventTypeSeq const&) throw ()
{
  // We don't care.
}
